@@ -1,6 +1,6 @@
 /* Domain-independent part of the OCaml interface: non-inline functions.
    Copyright (C) 2001-2010 Roberto Bagnara <bagnara@cs.unipr.it>
-   Copyright (C) 2010-2011 BUGSENG srl (http://bugseng.com)
+   Copyright (C) 2010-2012 BUGSENG srl (http://bugseng.com)
 
 This file is part of the Parma Polyhedra Library (PPL).
 
@@ -19,7 +19,7 @@ along with this program; if not, write to the Free Software Foundation,
 Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1307, USA.
 
 For the most up-to-date information see the Parma Polyhedra Library
-site: http://www.cs.unipr.it/ppl/ . */
+site: http://bugseng.com/products/ppl/ . */
 
 #include "ppl_ocaml_common.defs.hh"
 
@@ -29,38 +29,30 @@ namespace Interfaces {
 
 namespace OCaml {
 
-#ifdef PPL_WATCHDOG_LIBRARY_ENABLED
-
-Parma_Watchdog_Library::Watchdog* p_timeout_object = 0;
+Parma_Polyhedra_Library::Watchdog* p_timeout_object = 0;
 
 typedef
-Parma_Watchdog_Library::Threshold_Watcher
+Parma_Polyhedra_Library::Threshold_Watcher
 <Parma_Polyhedra_Library::Weightwatch_Traits> Weightwatch;
 
 Weightwatch* p_deterministic_timeout_object = 0;
 
-#endif // PPL_WATCHDOG_LIBRARY_ENABLED
-
 void
 reset_timeout() {
-#ifdef PPL_WATCHDOG_LIBRARY_ENABLED
   if (p_timeout_object) {
     delete p_timeout_object;
     p_timeout_object = 0;
     abandon_expensive_computations = 0;
   }
-#endif // PPL_WATCHDOG_LIBRARY_ENABLED
 }
 
 void
 reset_deterministic_timeout() {
-#ifdef PPL_WATCHDOG_LIBRARY_ENABLED
   if (p_deterministic_timeout_object) {
     delete p_deterministic_timeout_object;
     p_deterministic_timeout_object = 0;
     abandon_expensive_computations = 0;
   }
-#endif // PPL_WATCHDOG_LIBRARY_ENABLED
 }
 
 namespace {
@@ -153,9 +145,7 @@ build_ppl_relsym(value caml_relsym) {
   case 4:
     return GREATER_THAN;
   default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error\n:"
-                             "build_ppl_relsym(rel)");
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_relsym(rel)");
   }
 }
 
@@ -174,9 +164,8 @@ build_ppl_bounded_integer_type_width(value caml_width) {
   case 4:
     return BITS_128;
   default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error\n:"
-                             "build_ppl_bounded_integer_type_width(width)");
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_bounded_integer_type_"
+                             "width(width)");
   }
 }
 
@@ -189,9 +178,8 @@ build_ppl_bounded_integer_type_representation(value caml_rep) {
   case 1:
     return SIGNED_2_COMPLEMENT;
   default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error\n:"
-                             "build_ppl_bounded_integer_type_representation(rep)");
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_bounded_integer_type_"
+                             "representation(rep)");
   }
 }
 
@@ -206,9 +194,8 @@ build_ppl_bounded_integer_type_overflow(value caml_oflow) {
   case 2:
     return OVERFLOW_IMPOSSIBLE;
   default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error\n:"
-                             "build_ppl_bounded_integer_type_overflow(oflow)");
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_bounded_integer_type_"
+                             "overflow(oflow)");
   }
 }
 
@@ -221,9 +208,7 @@ build_ppl_opt_mode(value caml_opt_mode) {
   case 1:
     return MAXIMIZATION;
   default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error\n:"
-                             "build_ppl_opt_mode(opt)");
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_opt_mode(opt)");
   }
 }
 
@@ -236,9 +221,7 @@ build_ppl_Degenerate_Element(value de) {
   case 1:
     return EMPTY;
   default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error:\n"
-                             "build_ppl_Degenerate_Element(de)");
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_Degenerate_Element(de)");
   }
 }
 
@@ -253,23 +236,18 @@ build_ppl_Complexity_Class(value cc) {
   case 2:
     return ANY_COMPLEXITY;
   default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error:\n"
-                             "build_ppl_Complexity_Class(cc)");
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_Complexity_Class(cc)");
   }
 }
 
 MIP_Problem::Control_Parameter_Name
-build_ppl_control_parameter_name(value caml_cp_name) {
+build_ppl_mip_problem_control_parameter_name(value caml_cp_name) {
   assert(Is_long(caml_cp_name));
-  switch (Int_val(caml_cp_name)) {
-  case 0:
-    return  MIP_Problem::PRICING;
-  default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error:\n"
-                             "build_ppl_control_parameter_name(cpn)");
-  }
+  if (Int_val(caml_cp_name) == 0)
+    return MIP_Problem::PRICING;
+  else
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_mip_problem_"
+                             "control_parameter_name(cpn)");
 }
 
 MIP_Problem::Control_Parameter_Value
@@ -283,9 +261,7 @@ build_ppl_control_parameter_value(value caml_cp_value) {
   case 2:
     return MIP_Problem::PRICING_TEXTBOOK;
   default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error:\n"
-                             "build_ppl_control_parameter_value(cpv)");
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_control_parameter_value(cpv)");
   }
 }
 
@@ -294,13 +270,11 @@ build_ppl_pip_problem_control_parameter_name(value caml_cp_name) {
   assert(Is_long(caml_cp_name));
   switch (Int_val(caml_cp_name)) {
   case 0:
-    return  PIP_Problem::CUTTING_STRATEGY;
+    return PIP_Problem::CUTTING_STRATEGY;
   case 1:
-    return  PIP_Problem::PIVOT_ROW_STRATEGY;
+    return PIP_Problem::PIVOT_ROW_STRATEGY;
   default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error:\n"
-                             "build_ppl_pip_problem_"
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_pip_problem_"
                              "control_parameter_name(cpn)");
   }
 }
@@ -320,9 +294,7 @@ build_ppl_pip_problem_control_parameter_value(value caml_cp_value) {
   case 4:
     return PIP_Problem::PIVOT_ROW_STRATEGY_MAX_COLUMN;
   default:
-    // We should not be here!
-    throw std::runtime_error("PPL OCaml interface internal error:\n"
-                             "build_ppl_pip_problem_"
+    PPL_OCAML_UNEXPECTED_MSG("build_ppl_pip_problem_"
                              "control_parameter_value(cpv)");
   }
 }
@@ -460,10 +432,8 @@ build_ocaml_generator(const Generator& ppl_generator) {
     Store_field(caml_generator, 1, build_ocaml_coefficient(divisor));
     CAMLreturn(caml_generator);
   }
-  default:
-    throw std::runtime_error("PPL OCaml interface internal error:\n"
-                             "build_ocaml_generator(g)");
-  }
+  } // switch (ppl_generator.type())
+  PPL_UNREACHABLE;
 }
 
 value
@@ -494,10 +464,8 @@ build_ocaml_grid_generator(const Grid_Generator& ppl_grid_generator) {
     Store_field(caml_generator, 1, build_ocaml_coefficient(divisor));
     CAMLreturn(caml_generator);
   }
-  default:
-    throw std::runtime_error("PPL OCaml interface internal error:\n"
-                             "build_ocaml_grid_generator(g)");
-  }
+  } // switch (ppl_grid_generator.type())
+  PPL_UNREACHABLE;
 }
 
 value
@@ -523,10 +491,8 @@ build_ocaml_constraint(const Constraint& ppl_constraint) {
     Store_field(caml_constraint, 1, get_inhomogeneous_term(ppl_constraint));
     CAMLreturn(caml_constraint);
   }
-  default:
-    throw std::runtime_error("PPL OCaml interface internal error:\n"
-                             "build_ocaml_constraint(c)");
-  }
+  } // switch (ppl_constraint.type())
+  PPL_UNREACHABLE;
 }
 
 value
@@ -770,7 +736,7 @@ custom_MIP_Problem_finalize(value v) {
 }
 
 static struct custom_operations MIP_Problem_custom_operations = {
-  "it.unipr.cs.ppl" "." PPL_VERSION "." "MIP_Problem",
+  "com.bugseng.ppl" "." PPL_VERSION "." "MIP_Problem",
   custom_MIP_Problem_finalize,
   custom_compare_default,
   custom_hash_default,
@@ -798,7 +764,7 @@ custom_PIP_Problem_finalize(value v) {
 }
 
 static struct custom_operations PIP_Problem_custom_operations = {
-  "it.unipr.cs.ppl" "." PPL_VERSION "." "PIP_Problem",
+  "com.bugseng.ppl" "." PPL_VERSION "." "PIP_Problem",
   custom_PIP_Problem_finalize,
   custom_compare_default,
   custom_hash_default,
@@ -820,7 +786,7 @@ unregistered_value_p_PIP_Problem(const PIP_Problem& ph) {
 // In principle, we could directly store a PIP_Tree_Node* in a Caml 'value':
 // the use of a Custom_tag block is anyway recommended by Caml manual.
 static struct custom_operations PIP_Tree_Node_custom_operations = {
-  "it.unipr.cs.ppl" "." PPL_VERSION "." "PIP_Tree_Node",
+  "com.bugseng.ppl" "." PPL_VERSION "." "PIP_Tree_Node",
   custom_finalize_default,
   custom_compare_default,
   custom_hash_default,
@@ -1008,11 +974,8 @@ ppl_MIP_Problem_solve(value caml_mip) try {
     CAMLreturn(Val_int(1));
   case OPTIMIZED_MIP_PROBLEM:
     CAMLreturn(Val_int(2));
-  default:
-    ;
   }
-  // We should not be here!
-  throw std::runtime_error("PPL OCaml interface internal error");
+  PPL_UNREACHABLE;
 }
 CATCH_ALL
 
@@ -1027,11 +990,8 @@ ppl_MIP_Problem_optimization_mode(value caml_mip) try {
     CAMLreturn(Val_int(0));
   case MAXIMIZATION:
     CAMLreturn(Val_int(1));
-  default:
-    ;
   }
-  // We should not be here!
-  throw std::runtime_error("PPL OCaml interface internal error");
+  PPL_UNREACHABLE;
  }
 CATCH_ALL
 
@@ -1055,7 +1015,7 @@ ppl_MIP_Problem_get_control_parameter(value caml_mip,
   CAMLparam2(caml_mip, caml_cp_name);
   MIP_Problem& ppl_mip = *p_MIP_Problem_val(caml_mip);
   MIP_Problem::Control_Parameter_Name ppl_cp_name
-    = build_ppl_control_parameter_name(caml_cp_name);
+    = build_ppl_mip_problem_control_parameter_name(caml_cp_name);
   MIP_Problem::Control_Parameter_Value ppl_cp_value
     = ppl_mip.get_control_parameter(ppl_cp_name);
   switch (ppl_cp_value) {
@@ -1065,11 +1025,8 @@ ppl_MIP_Problem_get_control_parameter(value caml_mip,
     CAMLreturn(Val_int(1));
   case MIP_Problem::PRICING_TEXTBOOK:
     CAMLreturn(Val_int(2));
-  default:
-    ;
   }
-  // We should not be here!
-  throw std::runtime_error("PPL OCaml interface internal error");
+  PPL_UNREACHABLE;
  }
 CATCH_ALL
 
@@ -1180,7 +1137,7 @@ ppl_MIP_Problem_swap(value caml_mip1, value caml_mip2) try{
   CAMLparam2(caml_mip1, caml_mip2);
   MIP_Problem& ppl_mip1 = *p_MIP_Problem_val(caml_mip1);
   MIP_Problem& ppl_mip2 = *p_MIP_Problem_val(caml_mip2);
-  ppl_mip1.swap(ppl_mip2);
+  swap(ppl_mip1, ppl_mip2);
   CAMLreturn(Val_unit);
 }
 CATCH_ALL
@@ -1374,11 +1331,8 @@ ppl_PIP_Problem_solve(value caml_pip) try {
     CAMLreturn(Val_int(0));
   case OPTIMIZED_PIP_PROBLEM:
     CAMLreturn(Val_int(1));
-  default:
-    ;
   }
-  // We should not be here!
-  throw std::runtime_error("PPL OCaml interface internal error");
+  PPL_UNREACHABLE;
 }
 CATCH_ALL
 
@@ -1417,10 +1371,9 @@ ppl_PIP_Problem_get_control_parameter(value caml_pip,
   case PIP_Problem::PIVOT_ROW_STRATEGY_MAX_COLUMN:
     CAMLreturn(Val_int(4));
   default:
-    ;
+    PPL_UNREACHABLE;
+    CAMLreturn(Val_int(5));
   }
-  // We should not be here!
-  throw std::runtime_error("PPL OCaml interface internal error");
  }
 CATCH_ALL
 
@@ -1469,7 +1422,7 @@ ppl_PIP_Problem_swap(value caml_pip1, value caml_pip2) try{
   CAMLparam2(caml_pip1, caml_pip2);
   PIP_Problem& ppl_pip1 = *p_PIP_Problem_val(caml_pip1);
   PIP_Problem& ppl_pip2 = *p_PIP_Problem_val(caml_pip2);
-  ppl_pip1.swap(ppl_pip2);
+  swap(ppl_pip1, ppl_pip2);
   CAMLreturn(Val_unit);
 }
 CATCH_ALL
@@ -1811,19 +1764,13 @@ extern "C"
 CAMLprim value
 ppl_set_timeout(value time) try {
   CAMLparam1(time);
-#ifndef PPL_WATCHDOG_LIBRARY_ENABLED
-  const char* what = "PPL OCaml interface usage error:\n"
-    "ppl_set_timeout: the PPL Watchdog library is not enabled.";
-  throw std::runtime_error(what);
-#else
   // In case a timeout was already set.
   reset_timeout();
   unsigned cpp_time = value_to_unsigned<unsigned>(time);
   static timeout_exception e;
-  using Parma_Watchdog_Library::Watchdog;
+  using Parma_Polyhedra_Library::Watchdog;
   p_timeout_object = new Watchdog(cpp_time, abandon_expensive_computations, e);
   CAMLreturn(Val_unit);
-#endif // PPL_WATCHDOG_LIBRARY_ENABLED
 }
 CATCH_ALL
 
@@ -1831,33 +1778,26 @@ extern "C"
 CAMLprim value
 ppl_reset_timeout(value unit) try {
   CAMLparam1(unit);
-#ifndef PPL_WATCHDOG_LIBRARY_ENABLED
-  throw std::runtime_error("PPL OCaml interface error:\n"
-                           "the PPL Watchdog library is not enabled.");
-#else
   reset_timeout();
   CAMLreturn(Val_unit);
-#endif // PPL_WATCHDOG_LIBRARY_ENABLED
 }
 CATCH_ALL
 
 extern "C"
 CAMLprim value
-ppl_set_deterministic_timeout(value weight) try {
-  CAMLparam1(weight);
-#ifndef PPL_WATCHDOG_LIBRARY_ENABLED
-  const char* what = "PPL OCaml interface usage error:\n"
-    "ppl_set_deterministic_timeout: the PPL Watchdog library is not enabled.";
-  throw std::runtime_error(what);
-#else
+ppl_set_deterministic_timeout(value unscaled_weight, value scale) try {
+  CAMLparam2(unscaled_weight, scale);
   // In case a timeout was already set.
   reset_deterministic_timeout();
-  unsigned cpp_weight = value_to_unsigned<unsigned>(weight);
+  unsigned long cpp_unscaled_weight
+    = value_to_unsigned<unsigned long>(unscaled_weight);
+  unsigned cpp_scale = value_to_unsigned<unsigned>(scale);
   static deterministic_timeout_exception e;
+  typedef Parma_Polyhedra_Library::Weightwatch_Traits Traits;
   p_deterministic_timeout_object
-    = new Weightwatch(cpp_weight, abandon_expensive_computations, e);
+    = new Weightwatch(Traits::compute_delta(cpp_unscaled_weight, cpp_scale),
+                      abandon_expensive_computations, e);
   CAMLreturn(Val_unit);
-#endif // PPL_WATCHDOG_LIBRARY_ENABLED
 }
 CATCH_ALL
 
@@ -1865,12 +1805,7 @@ extern "C"
 CAMLprim value
 ppl_reset_deterministic_timeout(value unit) try {
   CAMLparam1(unit);
-#ifndef PPL_WATCHDOG_LIBRARY_ENABLED
-  throw std::runtime_error("PPL OCaml interface error:\n"
-                           "the PPL Watchdog library is not enabled.");
-#else
   reset_deterministic_timeout();
   CAMLreturn(Val_unit);
-#endif // PPL_WATCHDOG_LIBRARY_ENABLED
 }
 CATCH_ALL
