@@ -15,14 +15,21 @@ var NAVTREE =
   ] ]
 ];
 
+var NAVTREEINDEX =
+[
+".html",
+"group__PPL__CXX__interface.html#ggaa88e9b9cb2588cbf5914695fc14e69a8ae0c660bfcbcd3d10c3e764e22f41d0a1",
+"interfaceppl__Linear__Expression__tag.html#ab348eeffa0a61124b880932c35d0188b",
+"interfaceppl__Polyhedron__tag.html#ada9021952b7c7ad4ca6fc4ff082ec9fe",
+];
 
-var navTreeIndex;
+var navTreeSubIndices = new Array();
 
 function getData(varName)
 {
   var i = varName.lastIndexOf('/');
   var n = i>=0 ? varName.substring(i+1) : varName;
-  return eval(n);
+  return eval(n.replace(/\-/g,'_'));
 }
 
 function stripPath(uri)
@@ -30,6 +37,14 @@ function stripPath(uri)
   return uri.substring(uri.lastIndexOf('/')+1);
 }
 
+function stripPath2(uri)
+{
+  var i = uri.lastIndexOf('/');
+  var s = uri.substring(i+1);
+  var m = uri.substring(0,i+1).match(/\/d\w\/d\w\w\/$/);
+  return m ? uri.substring(i-6) : s;
+}
+
 function getScript(scriptName,func,show)
 {
   var head = document.getElementsByTagName("head")[0]; 
@@ -255,11 +270,13 @@ function highlightAnchor()
   if (anchor.parent().attr('class')=='memItemLeft'){
     var rows = $('.memberdecls tr[class$="'+
                window.location.hash.substring(1)+'"]');
-    glowEffect(rows.children(),300);
+    glowEffect(rows.children(),300); // member without details
+  } else if (anchor.parents().slice(2).prop('tagName')=='TR') {
+    glowEffect(anchor.parents('div.memitem'),1000); // enum value
   } else if (anchor.parent().is(":header")) {
-    glowEffect(anchor.parent(),1000);
+    glowEffect(anchor.parent(),1000); // section header
   } else {
-    glowEffect(anchor.next(),1000);
+    glowEffect(anchor.next(),1000); // normal member
   }
 }
 
@@ -314,7 +331,8 @@ function showNode(o, node, index)
             showNode(o,node,index); // retry with child node expanded
           },true);
         } else {
-          if (o.toroot=="index.html" || n.childrenData) {
+          var rootBase = o.toroot.replace(/\..+$/, '');
+          if (rootBase=="index" || rootBase=="pages") {
             expandNode(o, n, true, true);
           }
           selectAndHighlight(n);
@@ -337,27 +355,41 @@ function getNode(o, po)
   }
 }
 
-function gotoNode(o,root,hash)
+function gotoNode(o,subIndex,root,hash,relpath)
 {
-  var nti = navTreeIndex[root+hash];
-  o.breadcrumbs = nti ? nti : navTreeIndex[root];
-  if (o.breadcrumbs==null) o.breadcrumbs = navTreeIndex["index.html"];
-  o.breadcrumbs.unshift(0);
-  showNode(o, o.node, 0);
+  var nti = navTreeSubIndices[subIndex][root+hash];
+  o.breadcrumbs = nti ? nti : navTreeSubIndices[subIndex][root];
+  if (!o.breadcrumbs && root!=NAVTREE[0][1]) { // fallback: show index
+    navTo(o,NAVTREE[0][1],"",relpath);
+    $('.item').removeClass('selected');
+    $('.item').removeAttr('id');
+  }
+  if (o.breadcrumbs) {
+    o.breadcrumbs.unshift(0); // add 0 for root node
+    showNode(o, o.node, 0);
+  }
 }
 
 function navTo(o,root,hash,relpath)
 {
-  if (navTreeIndex){
-    gotoNode(o,root,hash);
+  if (hash.match(/^#l\d+$/)) {
+    var anchor=$('a[name='+hash.substring(1)+']');
+    glowEffect(anchor.parent(),1000); // line number
+    hash=''; // strip line number anchors
+  }
+  var url=root+hash;
+  var i=-1;
+  while (NAVTREEINDEX[i+1]<=url) i++;
+  if (navTreeSubIndices[i]) {
+    gotoNode(o,i,root,hash,relpath)
   } else {
-    getScript(relpath+"navtreeindex",function(){
-      navTreeIndex = eval('NAVTREEINDEX');
-      if (navTreeIndex){
-        gotoNode(o,root,hash);
+    getScript(relpath+'navtreeindex'+i,function(){
+      navTreeSubIndices[i] = eval('NAVTREEINDEX'+i);
+      if (navTreeSubIndices[i]) {
+        gotoNode(o,i,root,hash,relpath);
       }
     },true);
-  } 
+  }
 }
 
 function initNavTree(toroot,relpath)
@@ -394,7 +426,7 @@ function initNavTree(toroot,relpath)
          $('.item').removeClass('selected');
          $('.item').removeAttr('id');
        }
-       var link=stripPath($(location).attr('pathname'));
+       var link=stripPath2($(location).attr('pathname'));
        navTo(o,link,$(location).attr('hash'),relpath);
      }
   })
