@@ -24,7 +24,10 @@ site: http://bugseng.com/products/ppl/ . */
 #include "ppl-config.h"
 #include "Constraint.defs.hh"
 #include "Variable.defs.hh"
+#include "Variables_Set.defs.hh"
 #include "Congruence.defs.hh"
+#include "math_utilities.defs.hh"
+
 #include <iostream>
 #include <sstream>
 #include <stdexcept>
@@ -53,65 +56,65 @@ PPL::Constraint::throw_dimension_incompatible(const char* method,
 
 PPL::Constraint
 PPL::Constraint::construct_epsilon_geq_zero() {
-  Linear_Expression e = Variable(0);
+  Linear_Expression e;
   Constraint c(e, NONSTRICT_INEQUALITY, NOT_NECESSARILY_CLOSED);
+  c.set_epsilon_coefficient(Coefficient_one());
+  PPL_ASSERT(c.OK());
   return c;
 }
 
-PPL::Constraint::Constraint(const Congruence& cg)
-  : Linear_Row(cg.is_equality()
-	       // Size includes extra column for the inhomogeneous term.
-	       ? (cg.space_dimension() + 1)
-	       : (throw_invalid_argument("Constraint(cg)",
-					 "congruence cg must be an equality."),
-		  0U),
-	       // Capacity also includes a column for the epsilon coefficient.
-	       compute_capacity(cg.space_dimension() + 2,
-                                Dense_Row::max_size()),
-	       Flags(NECESSARILY_CLOSED, LINE_OR_EQUALITY)) {
-  Constraint& c = *this;
-  // Copy coefficients and inhomogeneous term.
-  for (dimension_type i = cg.space_dimension() + 1; i-- > 0; )
-    c[i] = cg[i];
+PPL::Constraint::Constraint(const Congruence& cg, Representation r)
+  : expr(cg, r),
+    wrapped_expr(expr, false),
+    kind_(LINE_OR_EQUALITY),
+    topology_(NECESSARILY_CLOSED) {
+  if (!cg.is_equality())
+    throw_invalid_argument("Constraint(cg)",
+                           "congruence cg must be an equality.");
   // Enforce normalization.
   strong_normalize();
+  PPL_ASSERT(OK());
 }
 
-PPL::Constraint::Constraint(const Congruence& cg,
-			    dimension_type sz,
-			    dimension_type capacity)
-  : Linear_Row(cg.is_equality()
-	       ? sz
-	       : (throw_invalid_argument("Constraint(cg, sz, c)",
-					 "congruence cg must be an equality."),
-		  0U),
-	       capacity,
-	       Flags(NECESSARILY_CLOSED, LINE_OR_EQUALITY)) {
-  Constraint& c = *this;
-  // Copy coefficients.
-  PPL_ASSERT(sz > 0);
-  while (sz-- > 0)
-    c[sz] = cg[sz];
+void
+PPL::Constraint::swap_space_dimensions(Variable v1, Variable v2) {
+  PPL_ASSERT(v1.space_dimension() <= space_dimension());
+  PPL_ASSERT(v2.space_dimension() <= space_dimension());
+  expr.swap_space_dimensions(v1, v2);
+  // *this is still normalized but it may not be strongly normalized.
+  sign_normalize();
+  PPL_ASSERT(OK());
+}
+
+void
+PPL::Constraint
+::permute_space_dimensions(const std::vector<Variable>& cycle) {
+  if (cycle.size() < 2)
+    // No-op. No need to call sign_normalize().
+    return;
+
+  expr.permute_space_dimensions(cycle);
+  // *this is still normalized but may be not strongly normalized:
+  // sign normalization is necessary.
+  sign_normalize();
+  PPL_ASSERT(OK());
 }
 
 bool
 PPL::Constraint::is_tautological() const {
-  PPL_ASSERT(size() > 0);
-  const Constraint& x = *this;
-  if (x.all_homogeneous_terms_are_zero())
+  if (expr.all_homogeneous_terms_are_zero())
     if (is_equality())
-      return x[0] == 0;
+      return expr.inhomogeneous_term() == 0;
     else
       // Non-strict inequality constraint.
-      return x[0] >= 0;
+      return expr.inhomogeneous_term() >= 0;
   else
     // There is a non-zero homogeneous coefficient.
     if (is_necessarily_closed())
       return false;
     else {
       // The constraint is NOT necessarily closed.
-      const dimension_type eps_index = size() - 1;
-      const int eps_sign = sgn(x[eps_index]);
+      const int eps_sign = sgn(epsilon_coefficient());
       if (eps_sign > 0)
 	// We have found the constraint epsilon >= 0.
 	return true;
@@ -120,61 +123,70 @@ PPL::Constraint::is_tautological() const {
 	return false;
       else {
 	// Here the epsilon coefficient is negative: strict inequality.
-	if (x[0] <= 0)
+	if (expr.inhomogeneous_term() <= 0)
 	  // A strict inequality such as `lhs - k > 0',
 	  // where k is a non negative integer, cannot be trivially true.
 	  return false;
 	// Checking for another non-zero coefficient.
-	for (dimension_type i = eps_index; --i > 0; )
-	  if (x[i] != 0)
-	    return false;
-	// We have the inequality `k > 0',
-	// where k is a positive integer.
-	return true;
+        // If the check succeeds, we have the inequality `k > 0',
+        // where k is a positive integer.
+        return wrapped_expr.all_homogeneous_terms_are_zero();
       }
     }
 }
 
 bool
 PPL::Constraint::is_inconsistent() const {
-  PPL_ASSERT(size() > 0);
-  const Constraint& x = *this;
-  if (x.all_homogeneous_terms_are_zero())
+  if (expr.all_homogeneous_terms_are_zero())
     // The inhomogeneous term is the only non-zero coefficient.
     if (is_equality())
-      return x[0] != 0;
+      return expr.inhomogeneous_term() != 0;
     else
       // Non-strict inequality constraint.
-      return x[0] < 0;
+      return expr.inhomogeneous_term() < 0;
   else
     // There is a non-zero homogeneous coefficient.
     if (is_necessarily_closed())
       return false;
     else {
       // The constraint is NOT necessarily closed.
-      const dimension_type eps_index = size() - 1;
-      if (x[eps_index] >= 0)
+      if (epsilon_coefficient() >= 0)
 	// If positive, we have found the constraint epsilon >= 0.
 	// If zero, one of the `true' dimensions has a non-zero coefficient.
 	// In both cases, it is not trivially false.
 	return false;
       else {
 	// Here the epsilon coefficient is negative: strict inequality.
-	if (x[0] > 0)
+	if (expr.inhomogeneous_term() > 0)
 	  // A strict inequality such as `lhs + k > 0',
 	  // where k is a positive integer, cannot be trivially false.
 	  return false;
-	// Checking for another non-zero coefficient.
-	for (dimension_type i = eps_index; --i > 0; )
-	  if (x[i] != 0)
-	    return false;
-	// We have the inequality `k > 0',
-	// where k is zero or a negative integer.
-	return true;
+        // Checking for another non-zero coefficient.
+        // If the check succeeds, we have the inequality `k > 0',
+        // where k is a positive integer.
+        return wrapped_expr.all_homogeneous_terms_are_zero();
       }
     }
 }
 
+void
+PPL::Constraint::linear_combine(const Constraint& y, dimension_type i) {
+  expr.linear_combine(y.expr, i);
+  strong_normalize();
+}
+
+/*! \relates Parma_Polyhedra_Library::Constraint */
+int
+PPL::compare(const Constraint& x, const Constraint& y) {
+  const bool x_is_line_or_equality = x.is_line_or_equality();
+  const bool y_is_line_or_equality = y.is_line_or_equality();
+  if (x_is_line_or_equality != y_is_line_or_equality)
+    // Equalities (lines) precede inequalities (ray/point).
+    return y_is_line_or_equality ? 2 : -2;
+
+  return compare(x.expr, y.expr);
+}
+
 bool
 PPL::Constraint::is_equivalent_to(const Constraint& y) const {
   const Constraint& x = *this;
@@ -202,19 +214,31 @@ PPL::Constraint::is_equivalent_to(const Constraint& y) const {
     x_expr.normalize();
     y_expr.normalize();
     // ... and finally check for syntactic equality.
-    for (dimension_type i = x_space_dim + 1; i-- > 0; )
-      if (x_expr[i] != y_expr[i])
-	return false;
-    return true;
+    return x_expr.is_equal_to(y_expr);
   }
 
   // `x' and 'y' are of the same type and they are not strict inequalities;
   // thus, the epsilon-coefficient, if present, is zero.
   // It is sufficient to check for syntactic equality.
-  for (dimension_type i = x_space_dim + 1; i-- > 0; )
-    if (x[i] != y[i])
-      return false;
-  return true;
+  return x.expr.is_equal_to(y.expr);
+}
+
+bool
+PPL::Constraint::is_equal_to(const Constraint& y) const {
+  return expr.is_equal_to(y.expr) && kind_ == y.kind_ && topology() == y.topology();
+}
+
+void
+PPL::Constraint::sign_normalize() {
+  if (is_line_or_equality())
+    expr.sign_normalize();
+}
+
+bool
+PPL::Constraint::check_strong_normalized() const {
+  Constraint tmp = *this;
+  tmp.strong_normalize();
+  return compare(*this, tmp) == 0;
 }
 
 const PPL::Constraint* PPL::Constraint::zero_dim_false_p = 0;
@@ -260,31 +284,104 @@ PPL::Constraint::finalize() {
   epsilon_leq_one_p = 0;
 }
 
+void
+PPL::Constraint::ascii_dump(std::ostream& s) const {
+  expr.ascii_dump(s);
+
+  s << " ";
+  
+  switch (type()) {
+  case Constraint::EQUALITY:
+    s << "=";
+    break;
+  case Constraint::NONSTRICT_INEQUALITY:
+    s << ">=";
+    break;
+  case Constraint::STRICT_INEQUALITY:
+    s << ">";
+    break;
+  }
+  s << " ";
+  if (topology() == NECESSARILY_CLOSED)
+    s << "(C)";
+  else
+    s << "(NNC)";
+
+  s << "\n";
+}
+
+bool
+PPL::Constraint::ascii_load(std::istream& s) {
+  std::string str;
+  std::string str2;
+
+  expr.ascii_load(s);
+
+  if (!(s >> str))
+    return false;
+  if (str == "=")
+    set_is_equality();
+  else if (str == ">=" || str == ">")
+    set_is_inequality();
+  else
+    return false;
+
+  if (!(s >> str2))
+    return false;
+  if (str2 == "(NNC)") {
+    // TODO: Avoid the mark_as_*() methods if possible.
+    if (topology() == NECESSARILY_CLOSED)
+      mark_as_not_necessarily_closed();
+  } else
+    if (str2 == "(C)") {
+      // TODO: Avoid the mark_as_*() methods if possible.
+      if (topology() == NOT_NECESSARILY_CLOSED)
+        mark_as_necessarily_closed();
+    } else
+      return false;
+
+  // Checking for equality of actual and declared types.
+  switch (type()) {
+  case EQUALITY:
+    if (str != "=")
+      return false;
+    break;
+  case NONSTRICT_INEQUALITY:
+    if (str != ">=")
+      return false;
+    break;
+  case STRICT_INEQUALITY:
+    if (str != ">")
+      return false;
+    break;
+  }
+
+  return true;
+}
+
 /*! \relates Parma_Polyhedra_Library::Constraint */
 std::ostream&
 PPL::IO_Operators::operator<<(std::ostream& s, const Constraint& c) {
-  const dimension_type num_variables = c.space_dimension();
   PPL_DIRTY_TEMP_COEFFICIENT(cv);
   bool first = true;
-  for (dimension_type v = 0; v < num_variables; ++v) {
-    cv = c.coefficient(Variable(v));
-    if (cv != 0) {
-      if (!first) {
-	if (cv > 0)
-	  s << " + ";
-	else {
-	  s << " - ";
-	  neg_assign(cv);
-	}
+  for (Constraint::Expression::const_iterator i = c.expression().begin(),
+        i_end = c.expression().end(); i != i_end; ++i) {
+    cv = *i;
+    if (!first) {
+      if (cv > 0)
+        s << " + ";
+      else {
+        s << " - ";
+        neg_assign(cv);
       }
-      else
-	first = false;
-      if (cv == -1)
-	s << "-";
-      else if (cv != 1)
-	s << cv << "*";
-      s << PPL::Variable(v);
     }
+    else
+      first = false;
+    if (cv == -1)
+      s << "-";
+    else if (cv != 1)
+      s << cv << "*";
+    s << i.variable();
   }
   if (first)
     s << Coefficient_zero();
@@ -327,25 +424,18 @@ PPL_OUTPUT_DEFINITIONS(Constraint)
 
 bool
 PPL::Constraint::OK() const {
-  // Check the underlying Linear_Row object.
-  if (!Linear_Row::OK())
-    return false;
-
   // Topology consistency checks.
-  const dimension_type min_size = is_necessarily_closed() ? 1U : 2U;
-  if (size() < min_size) {
+  if (is_not_necessarily_closed() && expr.space_dimension() == 0) {
 #ifndef NDEBUG
     std::cerr << "Constraint has fewer coefficients than the minimum "
-	      << "allowed by its topology:"
-	      << std::endl
-	      << "size is " << size()
-	      << ", minimum is " << min_size << "."
+	      << "allowed by its topology."
 	      << std::endl;
 #endif
     return false;
   }
 
-  if (is_equality() && !is_necessarily_closed() && (*this)[size() - 1] != 0) {
+  if (is_equality() && is_not_necessarily_closed()
+      && epsilon_coefficient() != 0) {
 #ifndef NDEBUG
     std::cerr << "Illegal constraint: an equality cannot be strict."
 	      << std::endl;
