@@ -31,6 +31,9 @@ site: http://bugseng.com/products/ppl/ . */
 #include "Constraint_System.inlines.hh"
 #include "Generator.defs.hh"
 #include "Scalar_Products.defs.hh"
+#include "Scalar_Products.inlines.hh"
+#include "math_utilities.defs.hh"
+
 #include <stdexcept>
 #include <deque>
 #include <vector>
@@ -76,7 +79,7 @@ PPL::MIP_Problem::MIP_Problem(const dimension_type dim)
   : external_space_dim(dim),
     internal_space_dim(0),
     tableau(),
-    working_cost(0, Row_Flags()),
+    working_cost(0),
     mapping(),
     base(),
     status(PARTIALLY_SATISFIABLE),
@@ -105,7 +108,7 @@ PPL::MIP_Problem::MIP_Problem(const dimension_type dim,
   : external_space_dim(dim),
     internal_space_dim(0),
     tableau(),
-    working_cost(0, Row_Flags()),
+    working_cost(0),
     mapping(),
     base(),
     status(PARTIALLY_SATISFIABLE),
@@ -522,26 +525,21 @@ PPL::MIP_Problem
   //  - count the number of new slack variables.
   for (dimension_type i = cs_num_rows; i-- > first_pending_constraint; ) {
     const Constraint& cs_i = *(input_cs[i]);
-    bool found_a_nonzero_coeff = false;
-    bool found_many_nonzero_coeffs = false;
-    dimension_type nonzero_coeff_column_index = 0;
-    for (dimension_type sd = cs_i.space_dimension(); sd-- > 0; ) {
-      if (cs_i.coefficient(Variable(sd)) != 0) {
-        if (found_a_nonzero_coeff) {
-          found_many_nonzero_coeffs = true;
-          if (cs_i.is_inequality())
-            ++additional_slack_variables;
-          break;
-        }
-        else {
-          nonzero_coeff_column_index = sd + 1;
-          found_a_nonzero_coeff = true;
-        }
-      }
-    }
+    const dimension_type cs_i_end = cs_i.space_dimension() + 1;
+
+    dimension_type nonzero_coeff_column_index
+      = cs_i.expression().first_nonzero(1, cs_i_end);
+    bool found_a_nonzero_coeff = (nonzero_coeff_column_index != cs_i_end);
+    bool found_many_nonzero_coeffs
+      = (found_a_nonzero_coeff
+         && !cs_i.expression().all_zeroes(nonzero_coeff_column_index + 1,
+                                               cs_i_end));
+
     // If more than one coefficient is nonzero,
     // continue with next constraint.
     if (found_many_nonzero_coeffs) {
+      if (cs_i.is_inequality())
+        ++additional_slack_variables;
       // CHECKME: Is it true that in the first phase we can apply
       // `is_satisfied()' with the generator `point()'?  If so, the following
       // code works even if we do not have a feasible point.
@@ -707,7 +705,7 @@ PPL::MIP_Problem::process_pending_constraints() {
 
   // Resize the tableau: first add additional rows ...
   if (additional_tableau_rows > 0)
-    tableau.add_zero_rows(additional_tableau_rows, Row_Flags());
+    tableau.add_zero_rows(additional_tableau_rows);
 
   // ... then add additional columns.
   // We need columns for additional (split) problem variables, additional
@@ -778,16 +776,17 @@ PPL::MIP_Problem::process_pending_constraints() {
     Row::iterator itr = tableau_k.end();
 
     const Constraint& c = *(input_cs[i + first_pending_constraint]);
-    for (dimension_type sd = c.space_dimension(); sd-- > 0; ) {
-      Coefficient_traits::const_reference coeff_sd
-        = c.coefficient(Variable(sd));
-      if (coeff_sd != 0) {
-        itr = tableau_k.insert(itr, mapping[sd+1].first, coeff_sd);
-        // Split if needed.
-        if (mapping[sd+1].second != 0) {
-          itr = tableau_k.insert(itr, mapping[sd+1].second);
-          neg_assign(*itr, coeff_sd);
-        }
+    const Constraint::Expression& c_e = c.expression();
+    for (Constraint::Expression::const_iterator j = c_e.begin(), j_end = c_e.end();
+        j != j_end; ++j) {
+      Coefficient_traits::const_reference coeff_sd = *j;
+      const std::pair<dimension_type, dimension_type> mapped
+        = mapping[j.variable().space_dimension()];
+      itr = tableau_k.insert(itr, mapped.first, coeff_sd);
+      // Split if needed.
+      if (mapped.second != 0) {
+        itr = tableau_k.insert(itr, mapped.second);
+        neg_assign(*itr, coeff_sd);
       }
     }
     Coefficient_traits::const_reference inhomo
@@ -830,7 +829,7 @@ PPL::MIP_Problem::process_pending_constraints() {
   }
 
   // Reset the working cost function to have the right size.
-  working_cost = working_cost_type(tableau_num_cols, Row_Flags());
+  working_cost = working_cost_type(tableau_num_cols);
 
   // Set up artificial variables: these will have coefficient 1 in the
   // constraint, will enter the base and will have coefficient -1 in
@@ -894,32 +893,21 @@ PPL::MIP_Problem::process_pending_constraints() {
   // the problem is unbounded as soon as the cost function has
   // a variable with a positive coefficient.
   if (tableau_num_rows == 0) {
-    const dimension_type input_obj_function_size
-      = input_obj_function.space_dimension();
-    for (dimension_type i = input_obj_function_size; i-- > 0; )
-      // If a the value of a variable in the objective function is
-      // different from zero, the final status is unbounded.
-      // In the first part the variable is constrained to be greater or equal
-      // than zero.
-      if ((((input_obj_function.coefficient(Variable(i)) > 0
-             && opt_mode == MAXIMIZATION)
-            || (input_obj_function.coefficient(Variable(i)) < 0
-        && opt_mode == MINIMIZATION)) && mapping[i].second == 0)
-          // In the following case the variable is unconstrained.
-          || (input_obj_function.coefficient(Variable(i)) != 0
-              && mapping[i].second != 0)) {
-        // Ensure the right space dimension is obtained.
-        last_generator = point(0 * Variable(space_dimension() - 1));
-        status = UNBOUNDED;
-        return true;
-      }
+    if (is_unbounded_obj_function(input_obj_function, mapping, opt_mode)) {
+      // Ensure the right space dimension is obtained.
+      last_generator = point();
+      last_generator.set_space_dimension(space_dimension());
+      status = UNBOUNDED;
+      return true;
+    }
 
     // The problem is neither trivially unfeasible nor trivially unbounded.
     // The tableau was successful computed and the caller has to figure
     // out which case applies.
     status = OPTIMIZED;
     // Ensure the right space dimension is obtained.
-    last_generator = point(0 * Variable(space_dimension() - 1));
+    last_generator = point();
+    last_generator.set_space_dimension(space_dimension());
     PPL_ASSERT(OK());
     return true;
   }
@@ -1360,37 +1348,44 @@ PPL::MIP_Problem::linear_combine(Dense_Row& x,
   PPL_DIRTY_TEMP_COEFFICIENT(normalized_x_k);
   PPL_DIRTY_TEMP_COEFFICIENT(normalized_y_k);
   normalize2(x_k, y_k, normalized_x_k, normalized_y_k);
-  Sparse_Row::const_iterator j = y.begin();
-  Sparse_Row::const_iterator j_end = y.end();
-  dimension_type i;
-  for (i = 0; j != j_end; ++i) {
-    PPL_ASSERT(i < x_size);
-    PPL_ASSERT(j.index() >= i);
-    if (j.index() == i) {
-      if (i != k) {
-        Coefficient& x_i = x[i];
-        x_i *= normalized_y_k;
-        Coefficient_traits::const_reference y_i = *j;
-        sub_mul_assign(x_i, y_i, normalized_x_k);
-      }
-      ++j;
-    } else {
-      if (i != k)
-        x[i] *= normalized_y_k;
-    }
-  }
-  PPL_ASSERT(j == j_end);
-  for ( ; i < x_size; ++i)
-    if (i != k)
-      x[i] *= normalized_y_k;
 
-  x[k] = 0;
+  neg_assign(normalized_y_k);
+  Parma_Polyhedra_Library::linear_combine(x, y, normalized_y_k, normalized_x_k);
+
+  PPL_ASSERT(x[k] == 0);
+
   x.normalize();
   WEIGHT_ADD_MUL(83, x_size);
 }
 
 #endif // defined(PPL_USE_SPARSE_MATRIX)
 
+bool
+PPL::MIP_Problem::is_unbounded_obj_function(
+  const Linear_Expression& x,
+  const std::vector<std::pair<dimension_type, dimension_type> >& mapping,
+  Optimization_Mode optimization_mode) {
+
+  for (Linear_Expression::const_iterator i = x.begin(), i_end = x.end();
+        i != i_end; ++i) {
+    // If a the value of a variable in the objective function is
+    // different from zero, the final status is unbounded.
+    // In the first part the variable is constrained to be greater or equal
+    // than zero.
+    if (mapping[i.variable().space_dimension()].second != 0)
+      return true;
+    if (optimization_mode == MAXIMIZATION) {
+      if (*i > 0)
+        return true;
+    } else {
+      PPL_ASSERT(optimization_mode == MINIMIZATION);
+      if (*i < 0)
+        return true;
+    }
+  }
+  return false;
+}
+
 // See pages 42-43 of [PapadimitriouS98].
 void
 PPL::MIP_Problem::pivot(const dimension_type entering_var_index,
@@ -1794,21 +1789,8 @@ PPL::MIP_Problem::second_phase() {
     return;
 
   // Build the objective function for the second phase.
-  const dimension_type input_obj_function_sd
-    = input_obj_function.space_dimension();
-  Row new_cost(input_obj_function_sd + 1, Row_Flags());
-  {
-    // This will be used as a hint.
-    Row::iterator itr = new_cost.end();
-    for (dimension_type i = input_obj_function_sd; i-- > 0; ) {
-      Coefficient_traits::const_reference c
-        = input_obj_function.coefficient(Variable(i));
-      if (c != 0)
-        itr = new_cost.insert(itr, i + 1, c);
-    }
-    if (input_obj_function.inhomogeneous_term() != 0)
-      itr = new_cost.insert(itr, 0, input_obj_function.inhomogeneous_term());
-  }
+  Row new_cost;
+  input_obj_function.get_row(new_cost);
 
   // Negate the cost function if we are minimizing.
   if (opt_mode == MINIMIZATION)
@@ -1820,9 +1802,8 @@ PPL::MIP_Problem::second_phase() {
 
   // Substitute properly the cost function in the `costs' matrix.
   {
-    working_cost_type tmp_cost
-      = working_cost_type(cost_zero_size, cost_zero_size, new_cost.flags());
-    tmp_cost.m_swap(working_cost);
+    working_cost_type tmp_cost(cost_zero_size, cost_zero_size);
+    swap(tmp_cost, working_cost);
   }
 
   {
@@ -1892,14 +1873,12 @@ PPL::MIP_Problem
   // and `evaluating_point'.
   const dimension_type working_space_dim
     = std::min(ep_space_dim, input_obj_function.space_dimension());
-  // Compute the optimal value of the cost function.
-  Coefficient_traits::const_reference divisor = evaluating_point.divisor();
-  numer = input_obj_function.inhomogeneous_term() * divisor;
-  for (dimension_type i = working_space_dim; i-- > 0; )
-    numer += evaluating_point.coefficient(Variable(i))
-      * input_obj_function.coefficient(Variable(i));
+  input_obj_function.scalar_product_assign(numer,
+                                           evaluating_point.expr,
+                                           0, working_space_dim + 1);
+
   // Numerator and denominator should be coprime.
-  normalize2(numer, divisor, numer, denom);
+  normalize2(numer, evaluating_point.divisor(), numer, denom);
 }
 
 bool
@@ -1993,6 +1972,9 @@ PPL::MIP_Problem::solve_mip(bool& have_incumbent_solution,
   PPL_DIRTY_TEMP_COEFFICIENT(gcd);
   Coefficient_traits::const_reference p_divisor = p.divisor();
   dimension_type non_int_dim = mip.space_dimension();
+  // TODO: This can be optimized more, exploiting the (possible)
+  // sparseness of p, if the size of i_vars is expected to be greater than
+  // the number of nonzeroes in p in most cases.
   for (Variables_Set::const_iterator v_begin = i_vars.begin(),
 	 v_end = i_vars.end(); v_begin != v_end; ++v_begin) {
     gcd_assign(gcd, p.coefficient(Variable(*v_begin)), p_divisor);
@@ -2077,6 +2059,9 @@ PPL::MIP_Problem::choose_branching_variable(const MIP_Problem& mip,
     = last_generator.divisor();
   Variables_Set candidate_variables;
 
+  // TODO: This can be optimized more, exploiting the (possible)
+  // sparseness of last_generator, if the size of i_vars is expected to be
+  // greater than the number of nonzeroes in last_generator in most cases.
   PPL_DIRTY_TEMP_COEFFICIENT(gcd);
   for (Variables_Set::const_iterator v_it = i_vars.begin(),
          v_end = i_vars.end(); v_it != v_end; ++v_it) {
@@ -2100,21 +2085,32 @@ PPL::MIP_Problem::choose_branching_variable(const MIP_Problem& mip,
         || is_saturated(*(input_cs[i]), last_generator))
       satisfiable_constraints[i] = true;
 
-  dimension_type current_num_appearances = 0;
   dimension_type winning_num_appearances = 0;
 
+  std::vector<dimension_type>
+    num_appearances(candidate_variables.space_dimension(), 0);
+
   // For every candidate variable, check how many times this appear in the
   // active constraints.
+  for (dimension_type i = input_cs_num_rows; i-- > 0; ) {
+    if (!satisfiable_constraints[i])
+      continue;
+    // TODO: This can be optimized more, exploiting the (possible)
+    // sparseness of input_cs, if the size of candidate_variables is expected
+    // to be greater than the number of nonzeroes of most rows.
+    for (Variables_Set::const_iterator v_it = candidate_variables.begin(),
+            v_end = candidate_variables.end(); v_it != v_end; ++v_it) {
+      if (*v_it >= input_cs[i]->space_dimension())
+        break;
+      if (input_cs[i]->coefficient(Variable(*v_it)) != 0)
+        ++num_appearances[*v_it];
+    }
+  }
   for (Variables_Set::const_iterator v_it = candidate_variables.begin(),
          v_end = candidate_variables.end(); v_it != v_end; ++v_it) {
-    current_num_appearances = 0;
-    for (dimension_type i = input_cs_num_rows; i-- > 0; )
-      if (satisfiable_constraints[i]
-          && *v_it < input_cs[i]->space_dimension()
-          && input_cs[i]->coefficient(Variable(*v_it)) != 0)
-        ++current_num_appearances;
-    if (current_num_appearances >= winning_num_appearances) {
-      winning_num_appearances = current_num_appearances;
+    const dimension_type n = num_appearances[*v_it];
+    if (n >= winning_num_appearances) {
+      winning_num_appearances = n;
       branching_index = *v_it;
     }
   }
@@ -2157,6 +2153,9 @@ PPL::MIP_Problem::is_mip_satisfiable(MIP_Problem& mip,
     = choose_branching_variable(mip, i_vars, non_int_dim);
 #else
   PPL_DIRTY_TEMP_COEFFICIENT(gcd);
+  // TODO: This can be optimized more, exploiting the (possible)
+  // sparseness of p, if the size of i_vars is expected to be greater than
+  // the number of nonzeroes in p in most cases.
   for (Variables_Set::const_iterator v_begin = i_vars.begin(),
 	 v_end = i_vars.end(); v_begin != v_end; ++v_begin) {
     gcd_assign(gcd, p.coefficient(Variable(*v_begin)), p_divisor);
@@ -2247,11 +2246,7 @@ PPL::MIP_Problem::OK() const {
     return false;
   }
 
-  for (dimension_type i = input_cs_num_rows; i-- > 0; )
-    if (!input_cs[i]->OK())
-      return false;
-
-  if (!tableau.OK() || !input_obj_function.OK() || !last_generator.OK())
+  if (!tableau.OK() || !last_generator.OK())
     return false;
 
   // Constraint system should contain no strict inequalities.
@@ -2309,6 +2304,10 @@ PPL::MIP_Problem::OK() const {
     // in the solution found.
     if (!i_variables.empty()) {
       PPL_DIRTY_TEMP_COEFFICIENT(gcd);
+      // TODO: This can be optimized more, exploiting the (possible)
+      // sparseness of last_generator, if the size of i_variables is expected
+      // to be greater than the number of nonzeroes in last_generator in most
+      // cases.
       for (Variables_Set::const_iterator v_it = i_variables.begin(),
             v_end = i_variables.end(); v_it != v_end; ++v_it) {
         gcd_assign(gcd, last_generator.coefficient(Variable(*v_it)),
