@@ -31,63 +31,6 @@ site: http://bugseng.com/products/ppl/ . */
 
 namespace PPL = Parma_Polyhedra_Library;
 
-void
-PPL::Polyhedron::add_space_dimensions(Linear_System& sys1,
-				      Linear_System& sys2,
-				      Bit_Matrix& sat1,
-				      Bit_Matrix& sat2,
-				      dimension_type add_dim) {
-  PPL_ASSERT(sys1.topology() == sys2.topology());
-  PPL_ASSERT(sys1.num_columns() == sys2.num_columns());
-  PPL_ASSERT(add_dim != 0);
-  using std::swap;
-
-  sys1.add_zero_columns(add_dim);
-  dimension_type old_index = sys2.first_pending_row();
-  sys2.add_rows_and_columns(add_dim);
-  // The added rows are in the non-pending part.
-  sys2.set_index_first_pending_row(old_index + add_dim);
-
-  // The resulting saturation matrix will be as follows:
-  // from row    0    to      add_dim-1       : only zeroes
-  //          add_dim     add_dim+num_rows-1  : old saturation matrix
-
-  // In fact all the old generators saturate all the new constraints
-  // because the polyhedron has not been embedded in the new space.
-  sat1.resize(sat1.num_rows() + add_dim, sat1.num_columns());
-  // The old matrix is moved to the end of the new matrix.
-  for (dimension_type i = sat1.num_rows() - add_dim; i-- > 0; )
-    swap(sat1[i], sat1[i+add_dim]);
-  // Computes the "sat_c", too.
-  sat2.transpose_assign(sat1);
-
-  if (!sys1.is_necessarily_closed()) {
-    // Moving the epsilon coefficients to the new last column.
-    dimension_type new_eps_index = sys1.num_columns() - 1;
-    dimension_type old_eps_index = new_eps_index - add_dim;
-    // This swap preserves sortedness of `sys1'.
-    sys1.swap_columns(old_eps_index, new_eps_index);
-
-    // Try to preserve sortedness of `sys2'.
-    if (!sys2.is_sorted())
-      sys2.swap_columns(old_eps_index, new_eps_index);
-    else {
-      for (dimension_type i = sys2.num_rows(); i-- > add_dim; ) {
-	Linear_Row& r = sys2[i];
-	swap(r[old_eps_index], r[new_eps_index]);
-      }
-      // The upper-right corner of `sys2' contains the J matrix:
-      // swap coefficients to preserve sortedness.
-      for (dimension_type i = add_dim; i-- > 0; ++old_eps_index) {
-	Linear_Row& r = sys2[i];
-	swap(r[old_eps_index], r[old_eps_index + 1]);
-      }
-    }
-    // NOTE: since we swapped columns in both `sys1' and `sys2',
-    // no swapping is required for `sat1' and `sat2'.
-  }
-}
-
 void
 PPL::Polyhedron::add_space_dimensions_and_embed(dimension_type m) {
   // The space dimension of the resulting polyhedron should not
@@ -142,40 +85,12 @@ PPL::Polyhedron::add_space_dimensions_and_embed(dimension_type m) {
     }
     else {
       // Only constraints are up-to-date: no need to modify the generators.
-      con_sys.add_zero_columns(m);
-      // If the polyhedron is not necessarily closed,
-      // move the epsilon coefficients to the last column.
-      if (!is_necessarily_closed())
-	con_sys.swap_columns(space_dim + 1, space_dim + 1 + m);
+      con_sys.set_space_dimension(con_sys.space_dimension() + m);
     }
   else {
     // Only generators are up-to-date: no need to modify the constraints.
     PPL_ASSERT(generators_are_up_to_date());
-    gen_sys.add_rows_and_columns(m);
-    // The polyhedron does not support pending generators.
-    gen_sys.unset_pending_rows();
-    // If the polyhedron is not necessarily closed,
-    // move the epsilon coefficients to the last column.
-    if (!is_necessarily_closed()) {
-      // Try to preserve sortedness of `gen_sys'.
-      if (!gen_sys.is_sorted())
-	gen_sys.swap_columns(space_dim + 1, space_dim + 1 + m);
-      else {
-        using std::swap;
-	dimension_type old_eps_index = space_dim + 1;
-	dimension_type new_eps_index = old_eps_index + m;
-	for (dimension_type i = gen_sys.num_rows(); i-- > m; ) {
-	  Generator& r = gen_sys[i];
-	  swap(r[old_eps_index], r[new_eps_index]);
-	}
-	// The upper-right corner of `gen_sys' contains the J matrix:
-	// swap coefficients to preserve sortedness.
-	for (dimension_type i = m; i-- > 0; ++old_eps_index) {
-	  Generator& r = gen_sys[i];
-	  swap(r[old_eps_index], r[old_eps_index + 1]);
-	}
-      }
-    }
+    gen_sys.add_universe_rows_and_space_dimensions(m);
   }
   // Update the space dimension.
   space_dim += m;
@@ -243,40 +158,12 @@ PPL::Polyhedron::add_space_dimensions_and_project(dimension_type m) {
     }
     else {
       // Only constraints are up-to-date: no need to modify the generators.
-      con_sys.add_rows_and_columns(m);
-      // The polyhedron does not support pending constraints.
-      con_sys.unset_pending_rows();
-      // If the polyhedron is not necessarily closed,
-      // move the epsilon coefficients to the last column.
-      if (!is_necessarily_closed()) {
-	// Try to preserve sortedness of `con_sys'.
-	if (!con_sys.is_sorted())
-	  con_sys.swap_columns(space_dim + 1, space_dim + 1 + m);
-	else {
-          using std::swap;
-	  dimension_type old_eps_index = space_dim + 1;
-	  dimension_type new_eps_index = old_eps_index + m;
-	  for (dimension_type i = con_sys.num_rows(); i-- > m; ) {
-	    Constraint& r = con_sys[i];
-	    swap(r[old_eps_index], r[new_eps_index]);
-	  }
-	  // The upper-right corner of `con_sys' contains the J matrix:
-	  // swap coefficients to preserve sortedness.
-	  for (dimension_type i = m; i-- > 0; ++old_eps_index) {
-	    Constraint& r = con_sys[i];
-	    swap(r[old_eps_index], r[old_eps_index + 1]);
-	  }
-	}
-      }
+      con_sys.add_universe_rows_and_space_dimensions(m);
     }
   else {
     // Only generators are up-to-date: no need to modify the constraints.
     PPL_ASSERT(generators_are_up_to_date());
-    gen_sys.add_zero_columns(m);
-    // If the polyhedron is not necessarily closed,
-    // move the epsilon coefficients to the last column.
-    if (!is_necessarily_closed())
-      gen_sys.swap_columns(space_dim + 1, space_dim + 1 + m);
+    gen_sys.set_space_dimension(gen_sys.space_dimension() + m);
   }
   // Now we update the space dimension.
   space_dim += m;
@@ -332,52 +219,29 @@ PPL::Polyhedron::concatenate_assign(const Polyhedron& y) {
   // by leaving the old system of constraints in the upper left-hand side
   // and placing the constraints of `cs' in the lower right-hand side.
   // NOTE: here topologies agree, whereas dimensions may not agree.
-  dimension_type old_num_rows = con_sys.num_rows();
-  dimension_type old_num_columns = con_sys.num_columns();
   dimension_type added_rows = cs.num_rows();
 
   // We already dealt with the cases of an empty or zero-dim `y' polyhedron;
   // also, `cs' contains the low-level constraints, at least.
   PPL_ASSERT(added_rows > 0 && added_columns > 0);
 
-  con_sys.add_zero_rows_and_columns(added_rows, added_columns,
-				    Linear_Row::Flags(topology(),
-						      Linear_Row::RAY_OR_POINT_OR_INEQUALITY));
-  // Move the epsilon coefficient to the last column, if needed.
-  if (!is_necessarily_closed())
-    con_sys.swap_columns(old_num_columns - 1,
-			 old_num_columns - 1 + added_columns);
-  dimension_type cs_num_columns = cs.num_columns();
-  // Steal the constraints from `cs' and put them in `con_sys'
-  // using the right displacement for coefficients.
-  for (dimension_type i = added_rows; i-- > 0; ) {
-    Constraint& c_old = cs[i];
-    Constraint& c_new = con_sys[old_num_rows + i];
-    // Method `add_zero_rows_and_columns', by default, added inequalities.
-    if (c_old.is_equality())
-      c_new.set_is_equality();
-    // The inhomogeneous term is not displaced.
-    using std::swap;
-    swap(c_new[0], c_old[0]);
-    // All homogeneous terms (included the epsilon coefficient,
-    // if present) are displaced by `space_dim' columns.
-    for (dimension_type j = 1; j < cs_num_columns; ++j)
-      swap(c_old[j], c_new[space_dim + j]);
-  }
+  con_sys.set_space_dimension(con_sys.space_dimension() + added_columns);
 
   if (can_have_something_pending()) {
+    // Steal the constraints from `cs' and put them in `con_sys'
+    // using the right displacement for coefficients.
+    for (dimension_type i = 0; i < added_rows; ++i) {
+      cs.sys.rows[i].shift_space_dimensions(Variable(0), space_dim);
+      con_sys.insert_pending(cs.sys.rows[i], Recycle_Input());
+    }
+    cs.clear();
+
     // If `*this' can support pending constraints, then, since we have
     // resized the system of constraints, we must also add to the generator
     // system those lines corresponding to the newly added dimensions,
     // because the non-pending parts of `con_sys' and `gen_sys' must still
     // be a DD pair in minimal form.
-    gen_sys.add_rows_and_columns(added_columns);
-    gen_sys.set_sorted(false);
-    if (!is_necessarily_closed())
-      gen_sys.swap_columns(old_num_columns - 1,
-			   old_num_columns - 1 + added_columns);
-    // The added lines are not pending.
-    gen_sys.unset_pending_rows();
+    gen_sys.add_universe_rows_and_space_dimensions(added_columns);
     // Since we added new lines at the beginning of `x.gen_sys',
     // we also have to adjust the saturation matrix `sat_c'.
     // FIXME: if `sat_c' is not up-to-date, could not we directly update
@@ -398,11 +262,13 @@ PPL::Polyhedron::concatenate_assign(const Polyhedron& y) {
     set_constraints_pending();
   }
   else {
-    // The polyhedron cannot have pending constraints.
-    con_sys.unset_pending_rows();
-#if BE_LAZY
-    con_sys.set_sorted(false);
-#else
+    // Steal the constraints from `cs' and put them in `con_sys'
+    // using the right displacement for coefficients.
+    for (dimension_type i = 0; i < added_rows; ++i) {
+      cs.sys.rows[i].shift_space_dimensions(Variable(0), space_dim);
+      con_sys.insert(cs.sys.rows[i], Recycle_Input());
+    }
+#if !BE_LAZY
     con_sys.sort_rows();
 #endif
     clear_constraints_minimized();
@@ -457,29 +323,7 @@ PPL::Polyhedron::remove_space_dimensions(const Variables_Set& vars) {
     return;
   }
 
-  // For each variable to be removed, we fill the corresponding column
-  // by shifting left those columns that will not be removed.
-  Variables_Set::const_iterator vsi = vars.begin();
-  Variables_Set::const_iterator vsi_end = vars.end();
-  dimension_type dst_col = *vsi + 1;
-  dimension_type src_col = dst_col + 1;
-  for (++vsi; vsi != vsi_end; ++vsi) {
-    const dimension_type vsi_col = *vsi + 1;
-    // All columns in between are moved to the left.
-    while (src_col < vsi_col)
-      gen_sys.Dense_Matrix::swap_columns(dst_col++, src_col++);
-    ++src_col;
-  }
-  // Moving the remaining columns.
-  const dimension_type gen_sys_num_columns = gen_sys.num_columns();
-  while (src_col < gen_sys_num_columns)
-    gen_sys.Dense_Matrix::swap_columns(dst_col++, src_col++);
-
-  // The number of remaining columns is `dst_col'.
-  // Note that resizing also calls `set_sorted(false)'.
-  gen_sys.remove_trailing_columns(gen_sys_num_columns - dst_col);
-  // We may have invalid lines and rays now.
-  gen_sys.remove_invalid_lines_and_rays();
+  gen_sys.remove_space_dimensions(vars);
 
   // Constraints are not up-to-date and generators are not minimized.
   clear_constraints_up_to_date();
@@ -526,18 +370,7 @@ PPL::Polyhedron::remove_higher_space_dimensions(dimension_type new_dimension) {
     return;
   }
 
-  dimension_type new_num_cols = new_dimension + 1;
-  if (!is_necessarily_closed()) {
-    // The polyhedron is not necessarily closed: move the column
-    // of the epsilon coefficients to its new place.
-    gen_sys.swap_columns(gen_sys.num_columns() - 1, new_num_cols);
-    // The number of remaining columns is `new_dimension + 2'.
-    ++new_num_cols;
-  }
-  // Note that resizing also calls `set_sorted(false)'.
-  gen_sys.remove_trailing_columns(space_dim - new_dimension);
-  // We may have invalid lines and rays now.
-  gen_sys.remove_invalid_lines_and_rays();
+  gen_sys.set_space_dimension(new_dimension);
 
   // Constraints are not up-to-date and generators are not minimized.
   clear_constraints_up_to_date();
@@ -551,8 +384,6 @@ PPL::Polyhedron::remove_higher_space_dimensions(dimension_type new_dimension) {
 
 void
 PPL::Polyhedron::expand_space_dimension(Variable var, dimension_type m) {
-  // TODO: this implementation is _really_ an executable specification.
-
   // `var' should be one of the dimensions of the vector space.
   if (var.space_dimension() > space_dim)
     throw_dimension_incompatible("expand_space_dimension(v, m)", "v", var);
@@ -575,30 +406,27 @@ PPL::Polyhedron::expand_space_dimension(Variable var, dimension_type m) {
   // Add the required new dimensions.
   add_space_dimensions_and_embed(m);
 
-  const dimension_type src_d = var.id();
   const Constraint_System& cs = constraints();
-  Constraint_System new_constraints;
+  Constraint_System new_constraints(cs.topology());
   for (Constraint_System::const_iterator i = cs.begin(),
 	 cs_end = cs.end(); i != cs_end; ++i) {
     const Constraint& c = *i;
 
+    Coefficient_traits::const_reference coeff = c.coefficient(var);
+
     // If `c' does not constrain `var', skip it.
-    if (c.coefficient(var) == 0)
+    if (coeff == 0)
       continue;
 
+    Constraint c_template = c;
+    c_template.expr.set_coefficient(var, Coefficient_zero());
+    
     // Each relevant constraint results in `m' new constraints.
     for (dimension_type dst_d = old_dim; dst_d < old_dim+m; ++dst_d) {
-      Linear_Expression e;
-      for (dimension_type j = old_dim; j-- > 0; )
-	e +=
-	  c.coefficient(Variable(j))
-	  * ((j == src_d) ? Variable(dst_d) : Variable(j));
-      e += c.inhomogeneous_term();
-      new_constraints.insert(c.is_equality()
-			     ? (e == 0)
-			     : (c.is_nonstrict_inequality()
-				? (e >= 0)
-				: (e > 0)));
+      Constraint new_c = c_template;
+      add_mul_assign(new_c.expr, coeff, Variable(dst_d));
+      PPL_ASSERT(new_c.OK());
+      new_constraints.insert(new_c, Recycle_Input());
     }
   }
   add_recycled_constraints(new_constraints);
